@@ -4,7 +4,7 @@
 ; dictionary.el written by Torsten Hilbrich <Torsten.Hilbrich@gmx.net>
 ; but now probably doesn't contain original code.
 ; Most of the code has been written
-; from scratch by Aleksey Cheusov <vle@gmx.net>, 2004.
+; from scratch by Aleksey Cheusov <vle@gmx.net>, 2004-2005.
 ;
 ; DictEm is free software; you can redistribute it and/or modify it
 ; under the terms of the GNU General Public License as published by
@@ -21,6 +21,10 @@
 ; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 ; 02111-1307, USA
 
+;
+; Note! Documentation is in README file.
+;
+
 (require 'cl)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -103,6 +107,14 @@ Instead, existing buffer will be erased and used to show results.
   :group 'dictem
   :type 'boolean)
 
+(defcustom dictem-use-content-history t
+  "If not nil and dictem-use-existing-buffer is also not nil,
+buffer content and (point) is saved in dictem-content-history variable
+when DEFINE hyperlinks are accessed.
+It is restored by dictem-last function.
+On slow machines it may better to set this variable to nil"
+  :group 'dictem)
+
 ;;;;;            Faces             ;;;;;
 
 (defface dictem-reference-definition-face
@@ -147,7 +159,7 @@ a single word in a MATCH search."
 ;;;;;           Variables          ;;;;;
 
 (defvar dictem-version
-  "0.0.2"
+  "0.0.5"
   "DictEm version information.")
 
 (defvar dictem-strategy-alist
@@ -216,7 +228,14 @@ by functions run from dictem-postprocess-each-definition-hook.")
 
 (defvar dictem-error-messages
   nil
-"A list of error messages collected by dictem-run")
+  "A list of error messages collected by dictem-run")
+
+(defvar dictem-hyperlinks-alist
+  nil
+  "ALIST of hyperlinks collected from dictem buffer by
+the function dictem-postprocess-collect-hyperlinks
+(add this function to the hook dictem-postprocess-definition-hook).
+This variable is local to buffer")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -440,7 +459,7 @@ and returns alist containing database names and descriptions"
 	"."))))
 
 (defun dictem-get-default-database (&optional def-db)
-  "Gets the default database"
+  "Returns the default database"
   (defun dictem-extract-dbname (database)
     (cond
      ((consp database) (dictem-extract-dbname (car database)))
@@ -482,10 +501,16 @@ and returns alist containing database names and descriptions"
   str)
 
 (defun dictem-remove-value-from-alist (l)
-  (cond
-   ((symbolp l) l)
-   (t (cons (list (caar l))
-	    (dictem-remove-value-from-alist (cdr l))))))
+  (let ((ret nil))
+    (dolist (i l)
+      (setq ret (cons (list (car i)) ret)))
+    (reverse ret)
+    ))
+;(defun dictem-remove-value-from-alist (l)
+;  (cond
+;   ((symbolp l) l)
+;   (t (cons (list (caar l))
+;	    (dictem-remove-value-from-alist (cdr l))))))
 
 (defun dictem-select (prompt alist default history)
   (let*
@@ -614,7 +639,7 @@ to enter a database name."
      'dictem-query-history default-query t)))
 
 
-;;;;;;;;    Search Functions     ;;;;;;;
+;;;;;;;;           Hooks        ;;;;;;;;
 
 (defcustom dictem-postprocess-definition-hook
   nil
@@ -623,7 +648,9 @@ to enter a database name."
   :type 'hook
   :options '(dictem-postprocess-definition-separator
 	     dictem-postprocess-definition-hyperlinks
-	     dictem-postprocess-each-definition))
+	     dictem-postprocess-each-definition
+	     dictem-postprocess-definition-remove-header
+	     dictem-postprocess-collect-hyperlinks))
 
 (defcustom dictem-postprocess-match-hook
   nil
@@ -637,7 +664,8 @@ to enter a database name."
   "Hook run in dictem mode buffers containing SHOW INFO result."
   :group 'dictem
   :type 'hook
-  :options '(dictem-postprocess-definition-hyperlinks))
+  :options '(dictem-postprocess-definition-hyperlinks
+	     dictem-postprocess-collect-hyperlinks))
 
 (defcustom dictem-postprocess-show-server-hook
   nil
@@ -645,6 +673,8 @@ to enter a database name."
   :group 'dictem
   :type 'hook)
 
+;;;;;;;;    Search Functions     ;;;;;;;
+
 (defun dictem-call-dict-internal (fun databases)
   (let ((exit-status -1))
 
@@ -758,6 +788,7 @@ to enter a database name."
 
   (setq dictem-last-database database)
   (setq dictem-last-strategy strategy)
+
   (dictem-call-dict-internal 'dictem-local-run-dict-search databases)))
 
 (defun dictem-base-define (databases query strategy)
@@ -765,131 +796,131 @@ to enter a database name."
   (interactive)
 
   (let ((ex_status -1))
-  (defun dictem-local-run-dict-define (database)
-    (let ((splitted-url nil))
-      (if (setq splitted-url (dictem-parse-url database))
-	  (save-dictem (setq dictem-server (nth 1 splitted-url))
-		       (setq dictem-port (dictem-get-port (nth 2 splitted-url)))
-		       (setq database      (nth 3 splitted-url))
-		       (dictem-initialize)
-		       (dictem-local-run-dict-define database)
-		       )
-	(let* ((beg (point))
-	       (exit_status
-		(call-process
-		 dictem-client-prog nil (current-buffer) nil
-		 "-P" "-" "-d" database
-		 "-h" (dictem-get-server) "-p" (dictem-get-port)
-		 "--client" (dictem-client-text)
-		 query)))
+    (defun dictem-local-run-dict-define (database)
+      (let ((splitted-url nil))
+	(if (setq splitted-url (dictem-parse-url database))
+	    (save-dictem (setq dictem-server (nth 1 splitted-url))
+			 (setq dictem-port (dictem-get-port (nth 2 splitted-url)))
+			 (setq database      (nth 3 splitted-url))
+			 (dictem-initialize)
+			 (dictem-local-run-dict-define database)
+			 )
+	  (let* ((beg (point))
+		 (exit_status
+		  (call-process
+		   dictem-client-prog nil (current-buffer) nil
+		   "-P" "-" "-d" database
+		   "-h" (dictem-get-server) "-p" (dictem-get-port)
+		   "--client" (dictem-client-text)
+		   query)))
 
-	  (cond ((= 0 exit_status)
-		 (save-excursion
-		   (narrow-to-region beg (point))
-		   (run-hooks 'dictem-postprocess-definition-hook)
-		   (widen)))
-		((= 21 exit_status)
-		 (save-excursion
-		   (narrow-to-region beg (point))
-		   (run-hooks 'dictem-postprocess-match-hook)
-		   (widen)))
-		(t
-		 (if (= -1 ex_status)
-		     (setq ex_status exit_status))
-		 (if (/= beg (point))
-		     (setq dictem-error-messages
-			   (append
-			    (list
-			     (dictem-make-url (dictem-get-server)
-					      (dictem-get-port) query t database)
-			     (buffer-substring-no-properties beg (point)))
-			    dictem-error-messages)))
-		 (kill-region beg (point))))
-	  ex_status))))
+	    (cond ((= 0 exit_status)
+		   (save-excursion
+		     (narrow-to-region beg (point))
+		     (run-hooks 'dictem-postprocess-definition-hook)
+		     (widen)))
+		  ((= 21 exit_status)
+		   (save-excursion
+		     (narrow-to-region beg (point))
+		     (run-hooks 'dictem-postprocess-match-hook)
+		     (widen)))
+		  (t
+		   (if (= -1 ex_status)
+		       (setq ex_status exit_status))
+		   (if (/= beg (point))
+		       (setq dictem-error-messages
+			     (append
+			      (list
+			       (dictem-make-url (dictem-get-server)
+						(dictem-get-port) query t database)
+			       (buffer-substring-no-properties beg (point)))
+			      dictem-error-messages)))
+		   (kill-region beg (point))))
+	    ex_status))))
 
-  (setq dictem-last-database database)
-  (dictem-call-dict-internal 'dictem-local-run-dict-define databases)))
+    (setq dictem-last-database database)
+    (dictem-call-dict-internal 'dictem-local-run-dict-define databases)))
 
 (defun dictem-base-match (databases query strategy)
   "dictem search: MATCH"
   (interactive)
 
   (let ((ex_status -1))
-  (defun dictem-local-run-dict-match (database)
-    (let ((splitted-url nil))
-      (if (setq splitted-url (dictem-parse-url database))
-	  (save-dictem (setq dictem-server (nth 1 splitted-url))
-		       (setq dictem-port (dictem-get-port (nth 2 splitted-url)))
-		       (setq database      (nth 3 splitted-url))
-		       (dictem-initialize)
-		       (dictem-local-run-dict-match database)
-		       )
-	(let* ((beg (point))
-	       (exit_status
-		(call-process
-		 dictem-client-prog nil (current-buffer) nil
-		 "-P" "-" "-d" database "-s" strategy
-		 "-h" (dictem-get-server) "-p" (dictem-get-port) "-m"
-		 "--client" (dictem-client-text)
-		 query)))
-	  (cond ((= 0 exit_status)
-		 (save-excursion
-		   (narrow-to-region beg (point))
-		   (run-hooks 'dictem-postprocess-match-hook)
-		   (widen)))
-		(t
-		 (if (= -1 ex_status)
-		     (setq ex_status exit_status))
-		 (if (/= beg (point))
-		     (setq dictem-error-messages
-			   (append
-			    (list
-			     (dictem-make-url (dictem-get-server)
-					      (dictem-get-port) query t database)
-			     (buffer-substring-no-properties beg (point)))
-			    dictem-error-messages)))
-		 (kill-region beg (point))))
-	  ex_status))))
+    (defun dictem-local-run-dict-match (database)
+      (let ((splitted-url nil))
+	(if (setq splitted-url (dictem-parse-url database))
+	    (save-dictem (setq dictem-server (nth 1 splitted-url))
+			 (setq dictem-port (dictem-get-port (nth 2 splitted-url)))
+			 (setq database      (nth 3 splitted-url))
+			 (dictem-initialize)
+			 (dictem-local-run-dict-match database)
+			 )
+	  (let* ((beg (point))
+		 (exit_status
+		  (call-process
+		   dictem-client-prog nil (current-buffer) nil
+		   "-P" "-" "-d" database "-s" strategy
+		   "-h" (dictem-get-server) "-p" (dictem-get-port) "-m"
+		   "--client" (dictem-client-text)
+		   query)))
+	    (cond ((= 0 exit_status)
+		   (save-excursion
+		     (narrow-to-region beg (point))
+		     (run-hooks 'dictem-postprocess-match-hook)
+		     (widen)))
+		  (t
+		   (if (= -1 ex_status)
+		       (setq ex_status exit_status))
+		   (if (/= beg (point))
+		       (setq dictem-error-messages
+			     (append
+			      (list
+			       (dictem-make-url (dictem-get-server)
+						(dictem-get-port) query t database)
+			       (buffer-substring-no-properties beg (point)))
+			      dictem-error-messages)))
+		   (kill-region beg (point))))
+	    ex_status))))
 
-  (setq dictem-last-database database)
-  (setq dictem-last-strategy strategy)
-  (dictem-call-dict-internal 'dictem-local-run-dict-match databases)))
+    (setq dictem-last-database database)
+    (setq dictem-last-strategy strategy)
+    (dictem-call-dict-internal 'dictem-local-run-dict-match databases)))
 
 (defun dictem-base-show-info (databases b c)
   "dictem: SHOW INFO command"
   (interactive)
 
   (let ((ex_status -1))
-  (defun run-dict-show-info (database)
-    (let* ((beg (point))
-	   (exit_status
-	    (call-process
-	     dictem-client-prog nil (current-buffer) nil
-	     "-P" "-" "-i" database
-	     "-h" (dictem-get-server) "-p" (dictem-get-port)
-	     "--client" (dictem-client-text)
-	     )))
-      (cond ((= 0 exit_status)
-	     (save-excursion
-	       (narrow-to-region beg (point))
-	       (run-hooks 'dictem-postprocess-show-info-hook)
-	       (widen)))
-	    (t
-	     (if (= -1 ex_status)
-		 (setq ex_status exit_status))
-	     (if (/= beg (point))
-		 (setq dictem-error-messages
-		       (append
-			(list
-			 (dictem-make-url (dictem-get-server)
-					  (dictem-get-port) "" t database)
-			 (buffer-substring-no-properties beg (point)))
-			dictem-error-messages)))
-	     (kill-region beg (point))))
-      ex_status))
+    (defun run-dict-show-info (database)
+      (let* ((beg (point))
+	     (exit_status
+	      (call-process
+	       dictem-client-prog nil (current-buffer) nil
+	       "-P" "-" "-i" database
+	       "-h" (dictem-get-server) "-p" (dictem-get-port)
+	       "--client" (dictem-client-text)
+	       )))
+	(cond ((= 0 exit_status)
+	       (save-excursion
+		 (narrow-to-region beg (point))
+		 (run-hooks 'dictem-postprocess-show-info-hook)
+		 (widen)))
+	      (t
+	       (if (= -1 ex_status)
+		   (setq ex_status exit_status))
+	       (if (/= beg (point))
+		   (setq dictem-error-messages
+			 (append
+			  (list
+			   (dictem-make-url (dictem-get-server)
+					    (dictem-get-port) "" t database)
+			   (buffer-substring-no-properties beg (point)))
+			  dictem-error-messages)))
+	       (kill-region beg (point))))
+	ex_status))
 
-  (setq dictem-last-database database)
-  (dictem-call-dict-internal 'run-dict-show-info databases)))
+    (setq dictem-last-database database)
+    (dictem-call-dict-internal 'run-dict-show-info databases)))
 
 (defun dictem-base-show-server (a b c)
   "dictem: SHOW SERVER command"
@@ -1005,6 +1036,8 @@ to enter a database name."
 	(set (make-local-variable 'dictem-use-user-databases-only) user-only)
 	(set (make-local-variable 'dictem-use-existing-buffer) use-existing-buf)
 
+	(set (make-local-variable 'dictem-hyperlinks-alist) nil)
+
 	;;;;;;;;;;;;;;
 	(setq case-replace nil)
 	(setq case-fold-search nil)
@@ -1034,39 +1067,41 @@ to enter a database name."
       (beginning-of-line)
     (goto-char (point-min))))
 
+(defun dictem-hyperlinks-menu ()
+  "Hyperlinks menu with autocompletion"
+  (interactive)
+  (let ((link (completing-read "Go to:" dictem-hyperlinks-alist)))
+    (if (and link (setq link (assoc link dictem-hyperlinks-alist)))
+	(dictem-run-define
+	 (cadr link)
+	 dictem-last-database))
+    ))
+
 (defun dictem-next-link ()
   "Move point to the next hyperlink"
   (interactive)
-
-  (defun dictem-local-next-link2 (limit)
-    (let ((pt nil))
-      (if (and (setq pt (next-single-property-change (point) 'link nil limit))
-	       (/= limit pt))
-	  (if (get-char-property pt 'link)
-	      (goto-char pt)
-	    (goto-char (next-single-property-change pt 'link nil limit))))
-      )
-    )
-
-  (dictem-local-next-link2 (point-max))
-  )
+  (let ((pt nil)
+	(limit (point-max)))
+    (if (and (setq pt (next-single-property-change
+		       (point) 'link nil limit))
+	     (/= limit pt))
+	(if (get-char-property pt 'link)
+	    (goto-char pt)
+	  (goto-char (next-single-property-change pt 'link nil limit))))
+    ))
 
 (defun dictem-previous-link ()
   "Move point to the previous hyperlink"
   (interactive)
-
-  (defun dictem-local-prev-link2 (limit)
-    (let ((pt nil))
-      (if (and (setq pt (previous-single-property-change (point) 'link nil limit))
-	       (/= limit pt))
-	  (if (get-char-property pt 'link)
-	      (goto-char pt)
-	    (goto-char (previous-single-property-change pt 'link nil limit))))
-      )
-    )
-
-  (dictem-local-prev-link2 (point-min))
-  )
+  (let ((pt nil)
+	(limit (point-min)))
+    (if (and (setq pt (previous-single-property-change
+		       (point) 'link nil limit))
+	     (/= limit pt))
+	(if (get-char-property pt 'link)
+	    (goto-char pt)
+	  (goto-char (previous-single-property-change pt 'link nil limit))))
+      ))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (defun dictem-help ()
@@ -1120,6 +1155,10 @@ The default key bindings:
   nil
   "The currently selected window")
 
+(defvar dictem-content-history
+  nil
+  "A list of lists (buffer_content point)")
+
 (defconst dictem-buffer-name
   "*dictem buffer*")
 
@@ -1151,6 +1190,7 @@ and returns a list containing protocol, server, port and path on nil if fails"
 
     (make-local-variable 'dictem-window-configuration)
     (make-local-variable 'dictem-selected-window)
+    (make-local-variable 'dictem-content-history)
     (setq dictem-window-configuration window-configuration)
     (setq dictem-selected-window selected-window)
     ))
@@ -1168,6 +1208,9 @@ and returns a list containing protocol, server, port and path on nil if fails"
 ; Bury the buffer
 (define-key dictem-mode-map "q" 'dictem-quit)
 
+; LAST, works like in Info-mode
+(define-key dictem-mode-map "l" 'dictem-last)
+
 ; Show help message
 (define-key dictem-mode-map "h" 'dictem-help)
 
@@ -1198,6 +1241,9 @@ and returns a list containing protocol, server, port and path on nil if fails"
 ; Move point to the previous HYPER LINK
 (define-key dictem-mode-map "\M-p" 'dictem-previous-link)
 
+; Hyperlinks menu
+(define-key dictem-mode-map "e" 'dictem-hyperlinks-menu)
+
 ; Scroll up dictem buffer
 (define-key dictem-mode-map " " 'scroll-up)
 
@@ -1222,6 +1268,11 @@ and returns a list containing protocol, server, port and path on nil if fails"
   "If current buffer is not a dictem buffer, create a new one."
   (if (dictem-mode-p)
       (progn
+	(if dictem-use-content-history
+	    (setq dictem-content-history
+		  (cons (list (buffer-substring
+			       (point-min) (point-max))
+			      (point)) dictem-content-history)))
 	(setq buffer-read-only nil)
 	(erase-buffer))
     (dictem)))
@@ -1248,21 +1299,31 @@ and returns a list containing protocol, server, port and path on nil if fails"
 		(select-window selected-window)
 		(set-window-configuration configuration)))))))
 
+(defun dictem-last ()
+  "Go back to the last buffer visited visited."
+  (interactive)
+  (if (eq major-mode 'dictem-mode)
+      (if dictem-content-history
+	  (progn
+	    (setq buffer-read-only nil)
+	    (delete-region (point-min) (point-max))
+	    (insert-string (car (car dictem-content-history)))
+	    (goto-char (cadr (car dictem-content-history)))
+	    (setq dictem-content-history (cdr dictem-content-history))
+	    )
+	  )
+    ))
+
 (defun dictem-kill-all-buffers ()
   "Kill all dictem buffers."
   (interactive)
-  (defun dictem-local-list-buffer-names (l)
-    (cond
-     (l
-      (let ((buf-name (buffer-name (car l))))
-	(if (and (<= (length dictem-buffer-name) (length buf-name))
-		 (string= dictem-buffer-name
-			  (substring buf-name 0 (length dictem-buffer-name))))
-	    (kill-buffer buf-name)))
-      (dictem-local-list-buffer-names (cdr l)))
-     ))
-
-  (dictem-local-list-buffer-names (buffer-list)))
+  (dolist (buffer (buffer-list))
+    (let ((buf-name (buffer-name buffer)))
+      (if (and (<= (length dictem-buffer-name) (length buf-name))
+	       (string= dictem-buffer-name
+			(substring buf-name 0 (length dictem-buffer-name))))
+	  (kill-buffer buf-name))
+      )))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;;;     Top-level Functions     ;;;;;;
@@ -1362,7 +1423,7 @@ link.  Upon clicking the `function' is called with `data' as argument."
     (remove-text-properties start end properties)
     (add-text-properties start end properties)))
 
-;;;;;;;   Coloring Functions     ;;;;;;;
+;;;;;;;   Postprocessing Functions     ;;;;;;;
 
 (defun dictem-postprocess-definition-separator ()
   (save-excursion
@@ -1392,43 +1453,104 @@ link.  Upon clicking the `function' is called with `data' as argument."
 	   (list (cons 'dbname dictem-current-dbname))))
 	))))
 
+(defvar dictem-hyperlink-beginning
+  "{"
+  "String that begins hyperlink.
+This variable is used by
+the function 'dictem-postprocess-definition-hyperlinks'")
+
+(defvar dictem-hyperlink-end
+  "}"
+  "String that ends hyperlink.
+This variable is used by
+the function 'dictem-postprocess-definition-hyperlinks'")
+
+(defvar dictem-hyperlink-define-func
+  'dictem-base-define
+  "Function called when user clicks on hyperlinks inside the definition.
+This variable is used by
+the function 'dictem-postprocess-definition-hyperlinks'")
+
+(defun dictem-postprocess-collect-hyperlinks ()
+  (save-excursion
+    (goto-char (point-min))
+    (let ((regexp (concat "\\(" dictem-hyperlink-beginning "\\([^{}|]+\\)"
+		  dictem-hyperlink-end
+		  "\\|\\(" dictem-hyperlink-beginning
+		  "\\([^{}|\n]+\\)|\\([^{}|\n]+\\)" dictem-hyperlink-end
+		  "\\)\\)")))
+
+      (while (search-forward-regexp regexp nil t)
+	(cond ((match-beginning 2)
+	       (let* ((word (dictem-replace-spaces
+			     (buffer-substring-no-properties
+			      (match-beginning 2)
+			      (match-end 2)))))
+		 (setq dictem-hyperlinks-alist
+		       (cons (list word word) dictem-hyperlinks-alist))
+		 ))
+	      ((match-beginning 3)
+	       (let* ((word-beg (match-beginning 4))
+		      (word-end (match-end 4))
+		      (link-beg (match-beginning 5))
+		      (link-end (match-end 5))
+		      (word (dictem-replace-spaces
+			     (buffer-substring-no-properties
+			      word-beg word-end)))
+		      (link (dictem-replace-spaces
+			     (buffer-substring-no-properties
+			      link-beg link-end)))
+		      )
+		 (setq dictem-hyperlinks-alist
+		       (cons (list word link) dictem-hyperlinks-alist))
+		 )))))
+    ))
+
 (defun dictem-postprocess-definition-hyperlinks ()
   (save-excursion
     (goto-char (point-min))
-    (let ((regexp "\\({[^{}|]+\\)}\\|^From [^\n]+\\[\\([^\n]+\\)\\]\\|\\({\\([^{}|\n]+\\)|\\([^{}|\n]+\\)}\\)"))
+    (let ((regexp
+	   (concat dictem-hyperlink-beginning "\\([^{}|]+\\)" dictem-hyperlink-end
+		   "\\|"
+		   "^From [^\n]+\\[\\([^\n]+\\)\\]"
+		   "\\|"
+		   "\\(" dictem-hyperlink-beginning "\\([^{}|\n]+\\)|\\([^{}|\n]+\\)"
+		   dictem-hyperlink-end "\\)")))
 
       (while (search-forward-regexp regexp nil t)
 	(cond ((match-beginning 1)
-	       (let* ((beg (+ 1 (match-beginning 1)))
+	       (let* ((beg (match-beginning 1))
 		      (end (match-end 1))
-		      (repl (buffer-substring beg end))
+		      (match-beg (match-beginning 0))
 		      (word (buffer-substring-no-properties beg end)))
-		 (replace-match repl)
+		 (replace-match word t t)
 		 (dictem-create-link
-		  (- beg 1) (- end 1)
+		  match-beg (+ match-beg (length word))
 		  'dictem-reference-definition-face
-		  'dictem-base-define
+		  dictem-hyperlink-define-func
 		  (list (cons 'word (dictem-replace-spaces word))
 			(cons 'dbname dictem-current-dbname))
 		  '(link t))
 		 ))
 	      ((match-beginning 2)
-	       (setq dictem-current-dbname
-		     (dictem-replace-spaces
-		      (buffer-substring-no-properties (match-beginning 2)
-						      (match-end 2)))))
+	       (if (null dictem-current-dbname)
+		   (setq dictem-current-dbname
+			 (dictem-replace-spaces
+			  (buffer-substring-no-properties (match-beginning 2)
+							  (match-end 2))))))
 	      ((match-beginning 3)
 	       (let* ((beg (match-beginning 5))
 		      (end (match-end 5))
+		      (match-beg (match-beginning 3))
 	              (repl-beg (match-beginning 4))
 		      (repl-end (match-end 4))
-		      (repl (buffer-substring repl-beg repl-end))
+		      (repl (buffer-substring-no-properties repl-beg repl-end))
 		      (word (buffer-substring-no-properties beg end)))
-		 (replace-match repl)
+		 (replace-match repl t t)
 		 (dictem-create-link
-		  (- repl-beg 1) (- repl-end 1)
+		  match-beg (+ match-beg (length repl))
 		  'dictem-reference-definition-face
-		  'dictem-base-define
+		  dictem-hyperlink-define-func
 		  (list (cons 'word (dictem-replace-spaces word))
 			(cons 'dbname dictem-current-dbname))
 		  '(link t))))
@@ -1473,6 +1595,19 @@ link.  Upon clicking the `function' is called with `data' as argument."
 		 (cons 'dbname last-database))))
 	 )))))
 
+(defun dictem-postprocess-definition-remove-header ()
+  (save-excursion
+    (goto-char (point-min))
+    (end-of-line)
+    (let (eol (point))
+      (goto-char (point-min))
+      (if (search-forward-regexp "definitions? found" eol t)
+	  (progn
+	    (goto-char (point-min))
+	    (kill-line 2)
+	    )
+	))))
+
 ;;;;;       On-Click Functions     ;;;;;
 
 (defun dictem-define-on-press ()
